package com.diagbot.facade;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.RuleTypeEnum;
import com.diagbot.enums.TypeEnum;
import com.diagbot.process.BillProcess;
import com.diagbot.rule.DrugRule;
import com.diagbot.rule.OtherRule;
import com.diagbot.util.MsgNewUtil;
import com.diagbot.util.RedisUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.Drug;
import com.diagbot.vo.ItemExt;
import com.diagbot.vo.RuleVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description: 开单合理性facade
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class BillFacade {

    @Autowired
    NeoFacade neoFacade;
    @Autowired
    BillProcess billProcess;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    RedisUtil redisUtil;
    @Autowired
    DrugRule drugRule;
    @Autowired
    MsgNewUtil msgNewUtil;
    @Autowired
    OtherRule otherRule;

    /**
     * 开单合理性业务
     *
     * @param
     * @return
     */
    public void billFac(WordCrfDTO wordCrfDTO, IndicationDTO res) {
        // 获取开单项入参
        List<RuleVO> ruleVOList = getRuleVO(wordCrfDTO);
        billProcess.processRule(ruleVOList, wordCrfDTO, res);

        // 【特殊规则】过敏原自身过敏
        otherRule.allergy(wordCrfDTO, res);
        // 【特殊规则】七院：非阻塞性睡眠呼吸暂停综合征且非混合性睡眠呼吸暂停综合征，不宜同时开立睡眠呼吸监测和眼电图（EOG）（双眼）
        otherRule.smhxjcAndYdteog(wordCrfDTO, res);

        // 结果去重处理
        commonFacade.dealMsg(res.getBillMsgList());
    }

    /**
     * 获取开单项入参
     *
     * @param wordCrfDTO
     * @return
     */
    public List<RuleVO> getRuleVO(WordCrfDTO wordCrfDTO) {
        List<RuleVO> ruleVOList = new ArrayList<>();
        // 药品
        List<Drug> drugOrder = wordCrfDTO.getDrugOrder();
        for (Drug item : drugOrder) {
            if (StringUtil.isNotBlank(item.getUniqueName())) {
                RuleVO ruleVO = new RuleVO();
                ruleVO.setInputName(item.getName());
                ruleVO.setLibName(item.getUniqueName());
                ruleVO.setLibType(LexiconEnum.Medicine.getKey());
                ruleVO.setLibTypeName(TypeEnum.drug.getName());
                ruleVO.setDateValue(item.getDateValue());
                ruleVO.setForm(item.getForm());
                ruleVO.setRouteName(item.getRouteName());
                ruleVO.setRouteUniqueName(item.getRouteUniqueName());
                ruleVOList.add(ruleVO);
            }
        }

        // 辅检
        List<Pacs> pacsOrder = wordCrfDTO.getPacsOrder();
        for (Pacs pacs : pacsOrder) {
            if (StringUtil.isNotBlank(pacs.getUniqueName())) {
                RuleVO ruleVO = new RuleVO();
                ruleVO.setInputName(pacs.getName());
                ruleVO.setLibName(pacs.getUniqueName());
                ruleVO.setLibType(LexiconEnum.PacsName.getKey());
                ruleVO.setLibTypeName(TypeEnum.pacs.getName());
                ruleVO.setDateValue(pacs.getDateValue());
                ruleVOList.add(ruleVO);

                RuleVO ruleVO1 = new RuleVO();
                ruleVO1.setInputName(pacs.getName());
                ruleVO1.setLibName(pacs.getUniqueName());
                ruleVO1.setLibType(LexiconEnum.PacsName.getKey());
                ruleVO1.setLibTypeName(TypeEnum.pacs.getName());
                ruleVO1.setSuffix(String.valueOf(RuleTypeEnum.repeat.getKey()));
                ruleVO1.setDateValue(pacs.getDateValue());
                ruleVOList.add(ruleVO1);
            }
        }

        // 化验套餐
        List<Lis> lisOrder = wordCrfDTO.getLisOrder();
        for (Lis lis : lisOrder) {
            if (StringUtil.isNotBlank(lis.getUniqueName())) {
                RuleVO ruleVO = new RuleVO();
                ruleVO.setInputName(lis.getName());
                ruleVO.setLibName(lis.getUniqueName());
                ruleVO.setLibType(LexiconEnum.LisName.getKey());
                ruleVO.setLibTypeName(TypeEnum.lis.getName());
                ruleVO.setDateValue(lis.getDateValue());
                ruleVOList.add(ruleVO);

                RuleVO ruleVO1 = new RuleVO();
                ruleVO1.setInputName(lis.getName());
                ruleVO1.setLibName(lis.getUniqueName());
                ruleVO1.setLibType(LexiconEnum.LisName.getKey());
                ruleVO1.setLibTypeName(TypeEnum.lis.getName());
                ruleVO1.setSuffix(String.valueOf(RuleTypeEnum.repeat.getKey()));
                ruleVO1.setDateValue(lis.getDateValue());
                ruleVOList.add(ruleVO1);
            }
        }

        // 手术和操作开单项
        List<Item> operations = wordCrfDTO.getOperationOrder();
        for (Item item : operations) {
            if (StringUtil.isNotBlank(item.getUniqueName())) {
                RuleVO ruleVO = new RuleVO();
                ruleVO.setInputName(item.getName());
                ruleVO.setLibName(item.getUniqueName());
                ruleVO.setLibType(LexiconEnum.Operation.getKey());
                ruleVO.setLibTypeName(TypeEnum.operation.getName());
                ruleVO.setDateValue(item.getDateValue());
                ruleVOList.add(ruleVO);
            }
        }

        // 输血开单项
        List<ItemExt> transfusions = wordCrfDTO.getTransfusionOrder();
        for (ItemExt item : transfusions) {
            if (StringUtil.isNotBlank(item.getUniqueName())) {
                RuleVO ruleVO = new RuleVO();
                ruleVO.setInputName(item.getName());
                ruleVO.setLibName(item.getUniqueName());
                ruleVO.setLibType(LexiconEnum.Transfusion.getKey());
                ruleVO.setLibTypeName(TypeEnum.transfusion.getName());
                ruleVO.setDateValue(item.getDateValue());
                ruleVOList.add(ruleVO);
            }
        }
        return ruleVOList;
    }
}
