package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.dto.ConceptDetailDTO;
import com.diagbot.dto.ConceptScaleDTO;
import com.diagbot.dto.ConceptScaleDetailDTO;
import com.diagbot.dto.ConceptScaleSubDTO;
import com.diagbot.dto.DictionaryInfoDTO;
import com.diagbot.dto.KlConceptDetailDTO;
import com.diagbot.dto.KlConceptStaticDTO;
import com.diagbot.dto.KlScaleByIdDTO;
import com.diagbot.dto.ScaleDTO;
import com.diagbot.dto.StaticKnowledgeDTO;
import com.diagbot.dto.StaticKnowledgeDetailDTO;
import com.diagbot.dto.StaticKnowledgeIndexPageDTO;
import com.diagbot.entity.KlConcept;
import com.diagbot.entity.KlConceptDetail;
import com.diagbot.entity.KlConceptScale;
import com.diagbot.entity.KlConceptStatic;
import com.diagbot.entity.KlRelation;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.KlConceptDetailService;
import com.diagbot.service.impl.KlConceptStaticServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ChangeStatusVO;
import com.diagbot.vo.GetDetailVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.KlConceptStaticPageVO;
import com.diagbot.vo.KlConceptStaticVO;
import com.diagbot.vo.KlScaleByIdVO;
import com.diagbot.vo.ScaleVO;
import com.diagbot.vo.StaticKnowledgeIndexPageVO;
import com.diagbot.vo.StaticKnowledgeVO;
import com.google.common.collect.Lists;
import io.micrometer.core.instrument.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/2/23 16:14
 */
@Component
public class KlConceptStaticFacade extends KlConceptStaticServiceImpl {
    @Autowired
    private KlConceptFacade klConceptFacade;
    @Autowired
    private KlRelationFacade klRelationFacade;
    @Autowired
    private KlConceptDetailFacade klConceptDetailFacade;
    @Autowired
    private KlConceptScaleFacade klConceptScaleFacade;
    @Autowired
    private KlDictionaryInfoFacade klDictionaryInfoFacade;
    @Autowired
    private KlScaleFacade klScaleFacade;

    @Autowired
    @Qualifier("klConceptDetailServiceImpl")
    private KlConceptDetailService klConceptDetailService;
    @Autowired
    private DictionaryFacade dictionaryFacade;
    @Autowired
    private MedRetrievalFacade medRetrievalFacade;
    @Autowired
    KlRegulationBaseFacade klRegulationBaseFacade;

    /**
     * 获取静态知识
     *
     * @param staticKnowledgeVO
     * @return
     */
    public StaticKnowledgeDTO getStaticKnowledge(StaticKnowledgeVO staticKnowledgeVO) {
        StaticKnowledgeDTO staticKnowledgeDTO = new StaticKnowledgeDTO();
        Integer type = convertType(staticKnowledgeVO.getType(), 1);
        if (type == null) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "请输入正确类型（1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作、8-量表、9-护理、10-政策法规）");
        }
        staticKnowledgeVO.setType(type);

        KlConcept concept = klConceptFacade.getOne(new QueryWrapper<KlConcept>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("lib_name", staticKnowledgeVO.getName())
                .eq("status", StatusEnum.Enable.getKey())
                .eq("lib_type", type), false);

        Integer sameType = 0;
        //标准词不存在，同名的是否存在，仅检验检查
        if (concept == null) {
            //同名静态知识互取
            if (type.equals(LexiconEnum.LisName.getKey())) {
                sameType = LexiconEnum.LisSubName.getKey();
            } else if (type.equals(LexiconEnum.LisSubName.getKey())) {
                sameType = LexiconEnum.LisName.getKey();
            } else if (type.equals(LexiconEnum.PacsName.getKey())) {
                sameType = LexiconEnum.PacsSubName.getKey();
            } else if (type.equals(LexiconEnum.PacsSubName.getKey())) {
                sameType = LexiconEnum.PacsName.getKey();
            }
            if (!sameType.equals(0)) {
                concept = klConceptFacade.getOne(new QueryWrapper<KlConcept>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq("lib_name", staticKnowledgeVO.getName())
                        .eq("lib_type", sameType)
                        .eq("status", StatusEnum.Enable.getKey()), false);
                if (concept == null) {
                    throw new CommonException(CommonErrorCode.NOT_EXISTS, "标准词不存在");
                }
            }
        }
        KlConceptStatic staticInfo = this.getOne(new QueryWrapper<KlConceptStatic>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId())
                .eq("status", StatusEnum.Enable.getKey()), false);
        if (staticInfo != null) {
            staticKnowledgeVO.setType(sameType);
        }

        //标准术语存在静态知识不存在，取同名静态知识，仅检验检查
        if (concept != null && staticInfo == null && sameType.equals(0)) {
            //同名静态知识互取
            if (type.equals(LexiconEnum.LisName.getKey())) {
                sameType = LexiconEnum.LisSubName.getKey();
            } else if (type.equals(LexiconEnum.LisSubName.getKey())) {
                sameType = LexiconEnum.LisName.getKey();
            } else if (type.equals(LexiconEnum.PacsName.getKey())) {
                sameType = LexiconEnum.PacsSubName.getKey();
            } else if (type.equals(LexiconEnum.PacsSubName.getKey())) {
                sameType = LexiconEnum.PacsName.getKey();
            }
            if (sameType != null) {
                KlConcept sameConcept = klConceptFacade.getOne(new QueryWrapper<KlConcept>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq("lib_name", staticKnowledgeVO.getName())
                        .eq("status", StatusEnum.Enable.getKey())
                        .eq("lib_type", sameType), false);
                if (sameConcept != null) {
                    staticInfo = this.getOne(new QueryWrapper<KlConceptStatic>()
                            .eq("is_deleted", IsDeleteEnum.N.getKey())
                            .eq("concept_id", sameConcept.getId())
                            .eq("status", StatusEnum.Enable.getKey()), false);
                    if (staticInfo != null) {
                        //存在同名的项，替换原来的项
                        BeanUtil.copyProperties(sameConcept, concept);
                        staticKnowledgeVO.setType(sameType);
                    } else {
                        //存在同名细项
                        if (sameType.equals(LexiconEnum.LisSubName.getKey())
                                || sameType.equals(LexiconEnum.PacsSubName.getKey())) {
                            BeanUtil.copyProperties(sameConcept, concept);
                            staticKnowledgeVO.setType(sameType);
                        }
                    }
                }
            }
        }

        if (staticInfo == null) {
            //检验子项或检查子项向上级取静态知识
            if (type.equals(LexiconEnum.LisSubName.getKey())
                    || type.equals(LexiconEnum.PacsSubName.getKey())
                    || sameType.equals(LexiconEnum.LisSubName.getKey())
                    || sameType.equals(LexiconEnum.PacsSubName.getKey())) {
                List<KlRelation> relations = klRelationFacade.list(new QueryWrapper<KlRelation>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq("end_id", concept.getId())
                        .eq("relation_id", 600));
                if (ListUtil.isNotEmpty(relations)) {
                    staticInfo = this.getOne(new QueryWrapper<KlConceptStatic>()
                            .eq("is_deleted", IsDeleteEnum.N.getKey())
                            .in("concept_id", relations.stream().map(i -> i.getStartId()).collect(Collectors.toList()))
                            .eq("status", StatusEnum.Enable.getKey()), false);
                    if (staticInfo == null) {
                        throw new CommonException(CommonErrorCode.NOT_EXISTS, "缺少" +
                                (type.equals(LexiconEnum.Scale.getKey()) ? "量表" : "静态") +
                                "信息");
                    }
                    concept = klConceptFacade.getById(staticInfo.getConceptId());
                } else {
                    throw new CommonException(CommonErrorCode.NOT_EXISTS, "缺少" +
                            (type.equals(LexiconEnum.Scale.getKey()) ? "量表" : "静态") +
                            "信息");
                }
            } else {
                throw new CommonException(CommonErrorCode.NOT_EXISTS, "缺少" +
                        (type.equals(LexiconEnum.Scale.getKey()) ? "量表" : "静态") +
                        "信息");
            }
        }


        //医学知识详情
        String sql = "";
        if (ListUtil.isNotEmpty(staticKnowledgeVO.getContentTypes())) {

            for (Integer contentType : staticKnowledgeVO.getContentTypes()) {
                if (contentType.equals(0)) {
                    sql = "";
                    break;
                }
                if (StringUtil.isNotBlank(sql)) {
                    sql += " or ";
                }
                sql += "find_in_set(" + contentType + ",content_type)";
            }
            if (StringUtil.isNotBlank(sql)) {
                sql = "(" + sql + ")";
            }
        }
        List<KlConceptDetail> details = klConceptDetailFacade.list(new QueryWrapper<KlConceptDetail>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId())
                .apply(sql)
                .orderByAsc("order_no"));

        List<StaticKnowledgeDetailDTO> detailList
                = BeanUtil.listCopyTo(details, StaticKnowledgeDetailDTO.class);
        List<StaticKnowledgeDetailDTO> introduceList = detailList
                .stream()
                .filter(i -> Arrays.asList(i.getContentType().split(",")).contains("1"))
                .collect(Collectors.toList());
        List<StaticKnowledgeDetailDTO> noticeList = detailList
                .stream()
                .filter(i -> Arrays.asList(i.getContentType().split(",")).contains("2"))
                .collect(Collectors.toList());
        List<StaticKnowledgeDetailDTO> clinicalPathwayList = detailList
                .stream()
                .filter(i -> Arrays.asList(i.getContentType().split(",")).contains("3"))
                .collect(Collectors.toList());
        List<StaticKnowledgeDetailDTO> treatInfoList = detailList
                .stream()
                .filter(i -> Arrays.asList(i.getContentType().split(",")).contains("4"))
                .collect(Collectors.toList());
        //Map<String, List<StaticKnowledgeDetailDTO>> detailMap = new HashMap<>();
        Map<String, List<StaticKnowledgeDetailDTO>> detailMap = new HashMap<>();
        if (ListUtil.isEmpty(staticKnowledgeVO.getContentTypes()) || staticKnowledgeVO.getContentTypes().contains(0)) {
            if (ListUtil.isNotEmpty(introduceList)) {
                detailMap.put("静态知识", introduceList);
            }
            if (ListUtil.isNotEmpty(noticeList)) {
                detailMap.put("注意事项", noticeList);
            }
            if (ListUtil.isNotEmpty(clinicalPathwayList)) {
                detailMap.put("临床路径", clinicalPathwayList);
            }
            if (ListUtil.isNotEmpty(treatInfoList)) {
                detailMap.put("治疗方案", treatInfoList);
            }
        } else {
            if (ListUtil.isNotEmpty(introduceList) && staticKnowledgeVO.getContentTypes().contains(1)) {
                detailMap.put("静态知识", introduceList);
            }
            if (ListUtil.isNotEmpty(noticeList) && staticKnowledgeVO.getContentTypes().contains(2)) {
                detailMap.put("注意事项", noticeList);
            }
            if (ListUtil.isNotEmpty(clinicalPathwayList) && staticKnowledgeVO.getContentTypes().contains(3)) {
                detailMap.put("临床路径", clinicalPathwayList);
            }
            if (ListUtil.isNotEmpty(treatInfoList) && staticKnowledgeVO.getContentTypes().contains(4)) {
                detailMap.put("治疗方案", treatInfoList);
            }
        }
        if (type.equals(LexiconEnum.Scale.getKey())) {
            //量表详情
            ScaleVO scaleVO = new ScaleVO();
            BeanUtil.copyProperties(staticKnowledgeVO, scaleVO);
            staticKnowledgeDTO.setScale(getScaleStructure(scaleVO, concept.getId()));
        }

        staticKnowledgeDTO.setClinicalPathwayName(staticInfo.getClinicalPathwayName());
        staticKnowledgeDTO.setNoticeName(staticInfo.getNoticeName());
        staticKnowledgeDTO.setDetails(detailMap);
        staticKnowledgeDTO.setId(concept.getId());
        staticKnowledgeDTO.setName(concept.getLibName());
        Integer retType = convertType(concept.getLibType(), 2);
        staticKnowledgeDTO.setType(retType == null ? "" : retType.toString());

        return staticKnowledgeDTO;
    }

    /**
     * 分页查询
     *
     * @param klConceptStaticPageVO
     * @return
     */
    @Override
    public IPage<KlConceptStaticDTO> getPage(KlConceptStaticPageVO klConceptStaticPageVO) {
        Integer type = convertType(klConceptStaticPageVO.getType(), 1);
        klConceptStaticPageVO.setType(type);

        List<DictionaryInfoDTO> dicTypeName = klDictionaryInfoFacade.getListByGroupType(13);
        Map<String, String> dicTypeNameMap
                = EntityUtil.makeMapWithKeyValue(dicTypeName, "val", "name");

        IPage<KlConceptStaticDTO> page = super.getPage(klConceptStaticPageVO);
        List<KlConceptStaticDTO> records = page.getRecords();
        List<Long> conceptIds = page.getRecords()
                .stream()
                .map(KlConceptStaticDTO::getId)
                .collect(Collectors.toList());
        if (ListUtil.isNotEmpty(conceptIds)) {
            QueryWrapper<KlConceptDetail> qw = new QueryWrapper<>();
            qw.in("concept_id", conceptIds)
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .orderByAsc("concept_id")
                    .orderByAsc("order_no");
            Map<Long, List<KlConceptDetail>> detailMap = klConceptDetailFacade.list(qw)
                    .stream().collect(Collectors.groupingBy(KlConceptDetail::getConceptId));

            records.forEach(record -> {
                String typeName = LexiconEnum.getName(record.getType());
                record.setTypeName(dicTypeNameMap.get(typeName));
                Integer retType = convertType(record.getType(), 2);
                record.setType(retType);
                if (null != detailMap
                        && detailMap.containsKey(record.getId())
                        && ListUtil.isNotEmpty(detailMap.get(record.getId()))) {
                    record.setTitle(detailMap.get(record.getId())
                            .stream()
                            .map(KlConceptDetail::getTitle)
                            .collect(Collectors.joining("、")));
                }
            });
        }
        page.setRecords(records);
        return page;
    }

    /**
     * 静态知识检索-分页返回
     *
     * @param staticKnowledgeIndexPageVO
     * @return
     */
    @Override
    public IPage<StaticKnowledgeIndexPageDTO> staticIndexPage(StaticKnowledgeIndexPageVO staticKnowledgeIndexPageVO) {
        //默认查找全部
        if (ListUtil.isEmpty(staticKnowledgeIndexPageVO.getTypes())) {
            staticKnowledgeIndexPageVO.setTypeIds(Arrays.asList(0));
        }
        staticKnowledgeIndexPageVO.setTypeIds(new ArrayList<>());

        List<Integer> types = staticKnowledgeIndexPageVO.getTypes();

        List<DictionaryInfoDTO> dicType = dictionaryFacade.getListByGroupType(10);
        Map<String, String> dicTypeMap = EntityUtil.makeMapWithKeyValue(dicType, "name", "val");
        List<DictionaryInfoDTO> dicTypeName = dictionaryFacade.getListByGroupType(13);

        //检索类型（多选）：0-全部、1-诊断、2-药品、3-检验、4-检查、5-手术和操作
        if (ListUtil.isEmpty(types)
                || (ListUtil.isNotEmpty(types) && types.contains(0))) {
            staticKnowledgeIndexPageVO.getTypeIds().addAll(Arrays.asList(LexiconEnum.Disease.getKey(),
                    LexiconEnum.Medicine.getKey(),
                    LexiconEnum.LisName.getKey(),
                    LexiconEnum.LisSubName.getKey(),
                    LexiconEnum.PacsName.getKey(),
                    LexiconEnum.PacsSubName.getKey(),
                    LexiconEnum.Operation.getKey(),
                    LexiconEnum.Scale.getKey(),
                    LexiconEnum.Nurse.getKey(),
                    LexiconEnum.Law.getKey()));
        } else {
            if (types.contains(1)) {
                staticKnowledgeIndexPageVO.getTypeIds().add(LexiconEnum.Disease.getKey());
            }
            if (types.contains(2)) {
                staticKnowledgeIndexPageVO.getTypeIds().add(LexiconEnum.Medicine.getKey());
            }
            if (types.contains(3)) {
                staticKnowledgeIndexPageVO.getTypeIds().add(LexiconEnum.LisName.getKey());
                staticKnowledgeIndexPageVO.getTypeIds().add(LexiconEnum.LisSubName.getKey());
            }
            if (types.contains(5)) {
                staticKnowledgeIndexPageVO.getTypeIds().add(LexiconEnum.PacsName.getKey());
                staticKnowledgeIndexPageVO.getTypeIds().add(LexiconEnum.PacsSubName.getKey());
            }
            if (types.contains(6)) {
                staticKnowledgeIndexPageVO.getTypeIds().add(LexiconEnum.Operation.getKey());
            }
            if (types.contains(8)) {
                staticKnowledgeIndexPageVO.getTypeIds().add(LexiconEnum.Scale.getKey());
            }
            if (types.contains(9)) {
                staticKnowledgeIndexPageVO.getTypeIds().add(LexiconEnum.Nurse.getKey());
            }
            if (types.contains(10)) {
                staticKnowledgeIndexPageVO.getTypeIds().add(LexiconEnum.Law.getKey());
            }
        }
        if (ListUtil.isEmpty(staticKnowledgeIndexPageVO.getTypeIds())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "请输入正确的检索类型（多选）：0-全部、1-诊断、2-药品、3-检验、4-检查、7-手术和操作、8-量表、9-护理、10-政策法规");
        }
        IPage<StaticKnowledgeIndexPageDTO> page = super.staticIndexPage(staticKnowledgeIndexPageVO);

        List<StaticKnowledgeIndexPageDTO> records = page.getRecords();

        if (ListUtil.isEmpty(records)) {
            return page;
        }
        List<Long> conceptIds = records.stream().map(StaticKnowledgeIndexPageDTO::getId).collect(Collectors.toList());
        List<KlConceptDetail> details = klConceptDetailFacade.list(new QueryWrapper<KlConceptDetail>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("concept_id", conceptIds)
                .orderByAsc("concept_id", "order_no"));
        Map<Long, List<KlConceptDetail>> detailMap = details.stream()
                .collect(Collectors.groupingBy(KlConceptDetail::getConceptId));

        List<KlConceptScale> scaleDetails = klConceptScaleFacade.list(new QueryWrapper<KlConceptScale>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("concept_id", conceptIds)
                .orderByAsc("text_type", "group_num", "order_no"));
        Map<Long, List<KlConceptScale>> scaleDetailMap = scaleDetails.stream()
                .collect(Collectors.groupingBy(KlConceptScale::getConceptId));

        records.forEach(i -> {
            i.setTypeName(medRetrievalFacade.convertTypeName(i.getTypeName(), 2, dicTypeName));
            if (dicTypeMap.containsKey(i.getTypeName())) {
                i.setType(Integer.valueOf(dicTypeMap.get(i.getTypeName())));
            }
            i.setHasInfo(1);
            if (detailMap != null
                    && detailMap.get(i.getId()) != null) {
                i.setHasStaticKnowledge(detailMap.get(i.getId()).stream()
                        .filter(x -> x.getContentType().contains("1")).count() > 0 ? 1 : 0);
                i.setHasNotice(detailMap.get(i.getId()).stream()
                        .filter(x -> x.getContentType().contains("2")).count() > 0 ? 1 : 0);
                i.setHasClinicalPathway(detailMap.get(i.getId()).stream()
                        .filter(x -> x.getContentType().contains("3")).count() > 0 ? 1 : 0);
                i.setHasTreatInfo(detailMap.get(i.getId()).stream()
                        .filter(x -> x.getContentType().contains("4")).count() > 0 ? 1 : 0);
                i.setAbstractContent(detailMap.get(i.getId()).get(0).getContent());
            } else {
                i.setHasStaticKnowledge(0);
                i.setHasNotice(0);
                i.setHasClinicalPathway(0);
                i.setHasTreatInfo(0);
            }
        });

        page.setRecords(records);
        return page;
    }

    /**
     * 保存记录（新增or修改）
     *
     * @param klConceptStaticVO
     * @return
     */
    public Boolean saveOrUpdateRecord(KlConceptStaticVO klConceptStaticVO) {
        if (klConceptStaticVO.getUserId() == null) {
            klConceptStaticVO.setUserId(0L);
        }
        Date now = DateUtil.now();

        Integer type = convertType(klConceptStaticVO.getType(), 1);
        klConceptStaticVO.setType(type);

        KlConcept concept = new KlConcept();
        if (klConceptStaticVO.getId() != null) {
            concept = klConceptFacade.getById(klConceptStaticVO.getId());
        } else {
            concept = klConceptFacade.getOne(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("lib_name", klConceptStaticVO.getName())
                    .eq("lib_type", klConceptStaticVO.getType()), false);
        }
        if (concept == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "标准术语不存在");
        }
        if (!concept.getLibName().equals(klConceptStaticVO.getName())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "标准术语名称不允许修改");
        }

        Long conceptId = concept.getId();
        KlConceptStatic staticInfo = this.getOne(new QueryWrapper<KlConceptStatic>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId()), false);

        if (staticInfo == null) {
            staticInfo = new KlConceptStatic();
            staticInfo.setId(null);
            staticInfo.setConceptId(concept.getId());
            staticInfo.setStatus(StatusEnum.Enable.getKey());
            staticInfo.setCreator(klConceptStaticVO.getUserId().toString());
            staticInfo.setGmtCreate(now);
        }
        staticInfo.setClinicalPathwayName(klConceptStaticVO.getClinicalPathwayName());
        staticInfo.setNoticeName(klConceptStaticVO.getNoticeName());
        staticInfo.setModifier(klConceptStaticVO.getUserId().toString());
        staticInfo.setGmtModified(now);

        List<KlConceptDetail> details = Lists.newLinkedList();
        if (ListUtil.isNotEmpty(klConceptStaticVO.getDetails())) {
            details = BeanUtil.listCopyTo(klConceptStaticVO.getDetails(), KlConceptDetail.class);
        }
        details.forEach(detail -> {
            detail.setConceptId(conceptId);
            detail.setCreator(klConceptStaticVO.getUserId().toString());
            detail.setGmtCreate(now);
            detail.setModifier(klConceptStaticVO.getUserId().toString());
            detail.setGmtModified(now);
        });

        //保存静态知识基础信息
        this.saveOrUpdate(staticInfo);

        //删除已有明细
        klConceptDetailService.remove(new QueryWrapper<KlConceptDetail>()
                .eq("concept_id", concept.getId()));

        //保存新的明细
        klConceptDetailService.saveBatch(details);

        //保存量表明细
        if (klConceptStaticVO.getSource() != null
                && klConceptStaticVO.getSource().equals(1)
                && klConceptStaticVO.getType().equals(LexiconEnum.Scale.getKey())) {
            if (klConceptStaticVO.getScale() != null) {
                klConceptStaticVO.getScale().setConceptId(concept.getId());
                klConceptStaticVO.getScale().setModifier(concept.getModifier());
                klScaleFacade.saveUpKlScaleAll(klConceptStaticVO.getScale());
            }
        }

        return true;
    }

    /**
     * 启用禁用
     *
     * @param changeStatusVO
     * @return
     */
    public Boolean changeStatus(ChangeStatusVO changeStatusVO) {
        //String userId = SysUserUtils.getCurrentPrincipleID();
        if (changeStatusVO.getUserId() == null) {
            changeStatusVO.setUserId(0L);
        }
        Date now = DateUtil.now();
        //更新主表
        this.update(new UpdateWrapper<KlConceptStatic>()
                .eq("concept_id", changeStatusVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .set("gmt_modified", now)
                .set("modifier", changeStatusVO.getUserId())
                .set("status", changeStatusVO.getStatus()));
        //更新明细表
        klConceptDetailService.update(new UpdateWrapper<KlConceptDetail>()
                .eq("concept_id", changeStatusVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .set("gmt_modified", now)
                .set("modifier", changeStatusVO.getUserId()));
        return true;
    }

    /**
     * 根据术语id获取静态信息
     *
     * @param idVO
     * @return
     */
    public KlConceptStaticDTO getRecordById(IdVO idVO) {
        KlConceptStaticDTO klConceptStaticDTO = new KlConceptStaticDTO();

        List<DictionaryInfoDTO> dicTypeName = klDictionaryInfoFacade.getListByGroupType(13);
        Map<String, String> dicTypeNameMap
                = EntityUtil.makeMapWithKeyValue(dicTypeName, "val", "name");

        KlConcept concept = klConceptFacade.getById(idVO.getId());
        if (concept == null) {
            return null;
        }

        KlConceptStatic staticInfo = this.getOne(new QueryWrapper<KlConceptStatic>()
                .eq("concept_id", idVO.getId()), false);

        if (staticInfo == null) {
            return null;
        }

        List<KlConceptDetail> details = klConceptDetailFacade.list(new QueryWrapper<KlConceptDetail>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", idVO.getId())
                .orderByAsc("order_no"));

        klConceptStaticDTO.setId(concept.getId());
        klConceptStaticDTO.setName(concept.getLibName());
        klConceptStaticDTO.setType(convertType(concept.getLibType(), 2));
        klConceptStaticDTO.setTypeName(dicTypeNameMap.get(LexiconEnum.getName(concept.getLibType())));

        klConceptStaticDTO.setStatus(staticInfo.getStatus());
        klConceptStaticDTO.setGmtModified(staticInfo.getGmtModified());
        klConceptStaticDTO.setModifier(staticInfo.getModifier());
        klConceptStaticDTO.setClinicalPathwayName(staticInfo.getClinicalPathwayName());
        klConceptStaticDTO.setNoticeName(staticInfo.getNoticeName());

        klConceptStaticDTO.setTitle(ListUtil.isNotEmpty(details)
                ? details.stream().map(i -> i.getTitle()).collect(Collectors.joining("、"))
                : "");
        klConceptStaticDTO.setDetails(ListUtil.isNotEmpty(details)
                ? BeanUtil.listCopyTo(details, KlConceptDetailDTO.class)
                : new ArrayList<>());

        //获取量表结构
        if (concept.getLibType().equals(LexiconEnum.Scale.getKey())) {
            KlScaleByIdVO klScaleByIdVO = new KlScaleByIdVO();
            klScaleByIdVO.setId(staticInfo.getId());
            List<KlScaleByIdDTO> scaleList = klScaleFacade.getKlScaleById(klScaleByIdVO);
            if (ListUtil.isNotEmpty(scaleList)) {
                klConceptStaticDTO.setScale(scaleList.get(0));
            }
        }
        return klConceptStaticDTO;
    }

    /**
     * 是否已存在
     *
     * @param klConceptStaticVO
     * @return
     */
    public Boolean isExist(KlConceptStaticVO klConceptStaticVO) {
        Integer type = convertType(klConceptStaticVO.getType(), 1);
        klConceptStaticVO.setType(type);

        KlConcept concept = new KlConcept();
        if (klConceptStaticVO.getId() != null) {
            concept = klConceptFacade.getById(klConceptStaticVO.getId());
        } else {
            concept = klConceptFacade.getOne(new QueryWrapper<KlConcept>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("lib_name", klConceptStaticVO.getName())
                    .eq("lib_type", type), false);
        }
        if (concept == null) {
            throw new CommonException(CommonErrorCode.NOT_EXISTS, "标准术语不存在");
        }
        if (!concept.getLibName().equals(klConceptStaticVO.getName())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR, "标准术语名称不允许修改");
        }

        KlConceptStatic staticInfo = this.getOne(new QueryWrapper<KlConceptStatic>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId()), false);

        if (staticInfo == null) {
            return false;
        }

        List<KlConceptDetail> details = klConceptDetailFacade.list(new QueryWrapper<KlConceptDetail>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId()));

        if (ListUtil.isEmpty(details)) {
            return false;
        }

        List<KlConceptScale> scales = klConceptScaleFacade.list(new QueryWrapper<KlConceptScale>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId()));
        return !ListUtil.isEmpty(scales);
    }

    /**
     * 页面展示类型转换
     *
     * @param type
     * @param flag
     * @return
     */
    public Integer convertType(Integer type, Integer flag) {
        Integer retType = null;
        if (type == null) {
            return retType;
        }
        if (flag.equals(1)) {
            //1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作
            switch (type) {
                case 1:
                    retType = LexiconEnum.Disease.getKey();
                    break;
                case 2:
                    retType = LexiconEnum.Medicine.getKey();
                    break;
                case 3:
                    retType = LexiconEnum.LisName.getKey();
                    break;
                case 4:
                    retType = LexiconEnum.LisSubName.getKey();
                    break;
                case 5:
                    retType = LexiconEnum.PacsName.getKey();
                    break;
                case 6:
                    retType = LexiconEnum.PacsSubName.getKey();
                    break;
                case 7:
                    retType = LexiconEnum.Operation.getKey();
                    break;
                case 8:
                    retType = LexiconEnum.Scale.getKey();
                    break;
                case 9:
                    retType = LexiconEnum.Nurse.getKey();
                    break;
                case 10:
                    retType = LexiconEnum.Law.getKey();
                    break;
                default:
                    break;
            }
        } else if (flag.equals(2)) {
            if (type.equals(LexiconEnum.Disease.getKey())) {
                retType = 1;
            } else if (type.equals(LexiconEnum.Medicine.getKey())) {
                retType = 2;
            } else if (type.equals(LexiconEnum.LisName.getKey())) {
                retType = 3;
            } else if (type.equals(LexiconEnum.LisSubName.getKey())) {
                retType = 4;
            } else if (type.equals(LexiconEnum.PacsName.getKey())) {
                retType = 5;
            } else if (type.equals(LexiconEnum.PacsSubName.getKey())) {
                retType = 6;
            } else if (type.equals(LexiconEnum.Operation.getKey())) {
                retType = 7;
            } else if (type.equals(LexiconEnum.Scale.getKey())) {
                retType = 8;
            } else if (type.equals(LexiconEnum.Nurse.getKey())) {
                retType = 9;
            } else if (type.equals(LexiconEnum.Law.getKey())) {
                retType = 10;
            }
        }
        return retType;
    }

    /**
     * 获取静态知识map Map<name_type,List<ConceptDetail>>
     *
     * @param getDetailVO
     * @return
     */
    public Map<String, List<ConceptDetailDTO>> getDetailByConcept(GetDetailVO getDetailVO) {
        Map<String, List<ConceptDetailDTO>> retMap = new HashMap<>();
        List<DictionaryInfoDTO> dicTypeName = klDictionaryInfoFacade.getListByGroupType(13);
        Map<String, String> dicTypeNameMap
                = EntityUtil.makeMapWithKeyValue(dicTypeName, "val", "name");

        QueryWrapper<KlConcept> conceptQueryWrapper = new QueryWrapper<>();
        conceptQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("status", 1);
        if (ListUtil.isNotEmpty(getDetailVO.getNameList())) {
            conceptQueryWrapper.in("lib_name", getDetailVO.getNameList());
        }
        List<KlConcept> conceptList = klConceptFacade.list(conceptQueryWrapper);
        Map<Long, KlConcept> conceptMap = EntityUtil.makeEntityMap(conceptList, "id");

        if (ListUtil.isNotEmpty(conceptList)) {
            List<Long> conceptIds = conceptList.stream()
                    .map(i -> i.getId())
                    .collect(Collectors.toList());

            List<KlConceptStatic> staticList = this.list(new QueryWrapper<KlConceptStatic>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("status", 1)
                    .in("concept_id", conceptIds));

            if (ListUtil.isEmpty(staticList)) {
                return retMap;
            }

            QueryWrapper<KlConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<>();
            conceptDetailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("concept_id", conceptIds);
            if (ListUtil.isNotEmpty(getDetailVO.getContentTypes())) {
                conceptDetailQueryWrapper.and(sql -> {
                    for (int i = 0; i < getDetailVO.getContentTypes().size(); i++) {
                        if (i > 0) {
                            sql.or();
                        }
                        sql.apply("find_in_set({0},content_type)", getDetailVO.getContentTypes().get(i));
                    }
                });
            }
            List<KlConceptDetail> conceptDetailList = klConceptDetailFacade.list(conceptDetailQueryWrapper);
            Map<Long, List<KlConceptDetail>> detailMap
                    = EntityUtil.makeEntityListMap(conceptDetailList, "conceptId");

            for (Map.Entry<Long, KlConcept> entry : conceptMap.entrySet()) {
                if (detailMap.containsKey(entry.getKey())) {
                    retMap.put(entry.getValue().getLibName() + "_" + dicTypeNameMap.get(LexiconEnum.getName(entry.getValue().getLibType())),
                            BeanUtil.listCopyTo(detailMap.get(entry.getKey()), ConceptDetailDTO.class));
                }
            }
        }
        return retMap;
    }

    /**
     * 获取量表结构map Map<name_type,List<KlConceptScale>>
     *
     * @param getDetailVO
     * @return
     */
    public Map<String, ConceptScaleDTO> getScaleDetailByConcept(GetDetailVO getDetailVO) {
        Map<String, ConceptScaleDTO> retMap = new HashMap<>();
        List<DictionaryInfoDTO> dicTypeName = klDictionaryInfoFacade.getListByGroupType(13);
        Map<String, String> dicTypeNameMap
                = EntityUtil.makeMapWithKeyValue(dicTypeName, "val", "name");

        QueryWrapper<KlConcept> conceptQueryWrapper = new QueryWrapper<>();
        conceptQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("status", 1)
                .eq("lib_type", LexiconEnum.Scale.getKey());
        if (ListUtil.isNotEmpty(getDetailVO.getNameList())) {
            conceptQueryWrapper.in("lib_name", getDetailVO.getNameList());
        }
        List<KlConcept> conceptList = klConceptFacade.list(conceptQueryWrapper);

        if (ListUtil.isNotEmpty(conceptList)) {
            for (KlConcept concept : conceptList) {
                retMap.put(concept.getLibName() + "_" + dicTypeNameMap.get(LexiconEnum.Scale.getName()),
                        getScaleStructure(new ScaleVO(), concept.getId()));
            }
        }
        return retMap;
    }

    /**
     * 获取量表结构
     *
     * @param conceptId
     * @return
     */
    public ConceptScaleDTO getScaleStructure(ScaleVO scaleVO, Long conceptId) {
        ConceptScaleDTO retDTO = new ConceptScaleDTO();
        List<KlConceptScale> scaleList = klConceptScaleFacade.list(new QueryWrapper<KlConceptScale>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", conceptId)
                .orderByAsc("text_type", "group_num", "order_no"));
        if (ListUtil.isEmpty(scaleList)) {
            return null;
        }
        List<ConceptScaleDetailDTO> scaleDetailList = BeanUtil.listCopyTo(scaleList, ConceptScaleDetailDTO.class);
        if (null != scaleVO) {
            List<String> codeList = new ArrayList<>();
            codeList = scaleDetailList.stream().filter(s -> StringUtils.isNotBlank(s.getRuleCode())).map(ac -> ac.getRuleCode()).collect(Collectors.toList());
            scaleVO.setCodeList(codeList);
            ScaleDTO scaleDTO = klRegulationBaseFacade.process(scaleVO);
            List<String> codeListNew = scaleDTO.getCodeList(); //Arrays.asList("100001", "100013");
            if (ListUtil.isNotEmpty(codeListNew)) {
                for (String str : codeListNew) {
                    for (ConceptScaleDetailDTO data2 : scaleDetailList) {
                        if (null != data2.getRuleCode() && data2.getRuleCode().equals(str)) {
                            data2.setMatch(1);
                        }
                    }
                }
            }
        }

        if (null != scaleVO) {
            List<String> codeList = new ArrayList<>();
            codeList = scaleDetailList.stream().map(ac -> ac.getRuleCode()).collect(Collectors.toList());
            scaleVO.setCodeList(codeList);
            ScaleDTO scaleDTO = klRegulationBaseFacade.process(scaleVO);
            List<String> codeListNew = scaleDTO.getCodeList(); //Arrays.asList("100001", "100013");
            if (ListUtil.isNotEmpty(codeListNew)) {
                for (String str : codeListNew) {
                    for (ConceptScaleDetailDTO data2 : scaleDetailList) {
                        if (null != data2.getRuleCode() && data2.getRuleCode().equals(str)) {
                            data2.setMatch(1);
                        }
                    }
                }
            }
        }

        Map<Long, List<ConceptScaleDetailDTO>> parentMap
                = scaleDetailList.stream().sorted(Comparator.comparing(ConceptScaleDetailDTO::getTextType).thenComparing(ConceptScaleDetailDTO::getGroupNum)).collect(Collectors.groupingBy(ConceptScaleDetailDTO::getParentId));
        List<ConceptScaleDetailDTO> retScacleDetailList = parentMap.get(-1L);
        if (ListUtil.isEmpty(retScacleDetailList)) {
            return null;
        }
        List<ConceptScaleSubDTO> retList = Lists.newLinkedList();
        Map<Integer, List<ConceptScaleDetailDTO>> groupMap
                = retScacleDetailList.stream().collect(Collectors.groupingBy(ConceptScaleDetailDTO::getGroupNum));
        for (Map.Entry<Integer, List<ConceptScaleDetailDTO>> entry : groupMap.entrySet()) {
            ConceptScaleSubDTO subItem = new ConceptScaleSubDTO();
            subItem.setGroupNum(entry.getKey());
            subItem.setDetailList(entry.getValue());
            retList.add(subItem);
        }
        for (ConceptScaleSubDTO subDTO : retList) {
            scaleRecursion(subDTO, parentMap);
        }
        //排序
        for (ConceptScaleSubDTO data : retList) {
            if (ListUtil.isNotEmpty(data.getDetailList())) {
                for (ConceptScaleDetailDTO datasub : data.getDetailList()) {
                    if (ListUtil.isNotEmpty(datasub.getSubList())) {
                        datasub.setSubList(datasub.getSubList().stream().sorted(Comparator.comparing(ConceptScaleSubDTO::getGroupNum)).collect(Collectors.toList()));
                    }

                }
            }
        }
        retDTO.setScaleDetails(retList);
        return retDTO;
    }

    /**
     * 递归
     *
     * @param item
     * @param treeMap
     */
    public ConceptScaleSubDTO scaleRecursion(ConceptScaleSubDTO item,
                                             Map<Long, List<ConceptScaleDetailDTO>> treeMap) {
        List<ConceptScaleDetailDTO> list = item.getDetailList();
        if (ListUtil.isNotEmpty(list)) {
            for (ConceptScaleDetailDTO detail : list) {
                List<ConceptScaleDetailDTO> subDetailList = treeMap.get(detail.getId());
                if (ListUtil.isNotEmpty(subDetailList)) {
                    Map<Integer, List<ConceptScaleDetailDTO>> subMap
                            = subDetailList.stream().collect(Collectors.groupingBy(ConceptScaleDetailDTO::getGroupNum));
                    List<ConceptScaleSubDTO> subList = Lists.newLinkedList();
                    for (Map.Entry<Integer, List<ConceptScaleDetailDTO>> subEntry : subMap.entrySet()) {
                        ConceptScaleSubDTO subDTO = new ConceptScaleSubDTO();
                        subDTO.setGroupNum(subEntry.getKey());
                        subDTO.setDetailList(subEntry.getValue());
                        scaleRecursion(subDTO, treeMap);
                        subList.add(subDTO);
                    }
                    detail.setSubList(subList);
                }
            }
        }
        return item;
    }
}