package com.diagbot.facade;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.dto.RegularValueDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.entity.KlRegularConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.RegularConfigEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.model.entity.AllergyMedicine;
import com.diagbot.model.entity.PD;
import com.diagbot.model.entity.PacsNum;
import com.diagbot.model.entity.Vital;
import com.diagbot.model.label.PacsLabel;
import com.diagbot.model.label.PastLabel;
import com.diagbot.model.label.VitalLabel;
import com.diagbot.service.impl.KlRegularConfigServiceImpl;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RegexUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.RegularConfigSaveVO;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections4.MapUtils;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @author wangfeng
 * @Description:
 * @date 2020-11-30 16:10
 */
@Component
public class KlRegularConfigFacade extends KlRegularConfigServiceImpl {

    public List<RegularValueDTO> getRegularConfigDatas(String text, List<KlRegularConfig> regularConfigs) {
        List<RegularValueDTO> listData = new ArrayList<>();
        if (StringUtil.isBlank(text)) {
            return Lists.newArrayList();
        }
        if (ListUtil.isNotEmpty(regularConfigs)) {
            for (KlRegularConfig data : regularConfigs) {
                String pattern = "";
                String patternMax = "";
                if (data.getRulesTepy().equals(RegularConfigEnum.allergy.getKey())) {
                    pattern = data.getRulesValue().split("￥")[0];
                    patternMax = data.getRulesValue().split("￥")[1];
                } else {
                    pattern = data.getRulesValue();
                }
                // 创建 Pattern 对象
                Pattern r = Pattern.compile(pattern, Pattern.CASE_INSENSITIVE);
                // 现在创建 matcher 对象
                Matcher m = r.matcher(text);
                int trueMin = 0;
                while (m.find()) {
                    //System.out.println(m.group());
                    String steing = m.group();
                    String value = "";
                    Integer type = data.getRulesTepy();
                    if (data.getRulesTepy().equals(RegularConfigEnum.lis.getKey())) {
                        value = steing.replaceAll("([^\\d\\.]|\\.(?=[^\\.]*\\.))", "");
                        RegularValueDTO regularValueDTO = new RegularValueDTO();
                        regularValueDTO.setKey(data.getRulesName());
                        regularValueDTO.setStandName(data.getStandName());
                        regularValueDTO.setType(type);
                        regularValueDTO.setValue(value);
                        listData.add(regularValueDTO);
                    } else if (data.getRulesTepy().equals(RegularConfigEnum.allergy.getKey())) {
                        //取否认的数
                        trueMin++;
                    }
                }
                int trueMax = 0;
                //取所有符合的
                if (data.getRulesTepy().equals(RegularConfigEnum.allergy.getKey())) {
                    Pattern rMax = Pattern.compile(patternMax, Pattern.CASE_INSENSITIVE);
                    // 现在创建 matcher 对象
                    Matcher mMax = rMax.matcher(text);
                    while (mMax.find()) {
                        if (data.getRulesTepy().equals(RegularConfigEnum.allergy.getKey())) {
                            trueMax++;
                        }
                    }
                    int mun = trueMax - trueMin;
                    if (mun > 0) {
                        RegularValueDTO regularValueDTO = new RegularValueDTO();
                        regularValueDTO.setKey(data.getRulesName());
                        regularValueDTO.setType(data.getRulesTepy());
                        regularValueDTO.setStandName(data.getStandName());
                        listData.add(regularValueDTO);
                    }
                }


            }


        }
        return listData;
    }

    /**
     * @param wordCrfDTO
     * @return
     */
    public WordCrfDTO getRegularConfigs(WordCrfDTO wordCrfDTO, IndicationPushVO indicationPushVO) {
        String allergyStr = indicationPushVO.getAllergy();
        String symptom = wordCrfDTO.getSymptom();//现病史
        String pasts = wordCrfDTO.getPasts();//既往史
        //辅检
        PacsLabel pacsLabel = wordCrfDTO.getPacsLabel();
        List<PacsNum> pacsNumList = pacsLabel.getPacsNumList();
        String pacs = pacsLabel.getText();
        //体征
        String vital = wordCrfDTO.getVital();
        VitalLabel vitalLabel = wordCrfDTO.getVitalLabel();
        List<Vital> vitals = vitalLabel.getVitals();

        if (StringUtil.isBlank(symptom) && StringUtil.isBlank(pasts) && StringUtil.isBlank(pacs)
        && StringUtil.isBlank(vital)) {
            return wordCrfDTO;
        }

        // 现病史化验和过敏史
        QueryWrapper<KlRegularConfig> regularQuery = new QueryWrapper<>();
        regularQuery.eq("is_deleted", IsDeleteEnum.N.getKey());
        List<KlRegularConfig> regularConfigs = list(regularQuery);
        List<RegularValueDTO> regularConfigDatas = getRegularConfigDatas(symptom, regularConfigs);
        List<Lis> lisData = wordCrfDTO.getLis();
        PastLabel pastLabel = wordCrfDTO.getPastLabel();//获取过敏史数据
        List<AllergyMedicine> allergies = pastLabel.getAllergyMedicines();
        if (ListUtil.isNotEmpty(regularConfigDatas)) {
            for (RegularValueDTO data : regularConfigDatas) {
                if (data.getType().equals(RegularConfigEnum.lis.getKey())) {
                    if (!data.getValue().equals("")) {
                        Lis lis = new Lis();
                        lis.setUniqueName(data.getKey());
                        lis.setDetailName(data.getKey());
                        lis.setName(data.getKey());
                        lis.setValue(Double.valueOf(data.getValue()));
                        lisData.add(lis);
                    }
                } else if (data.getType().equals(RegularConfigEnum.allergy.getKey())) {
                    AllergyMedicine allergy = new AllergyMedicine();
                    allergy.setName(data.getKey());
                    allergy.setStandName(data.getStandName());
                    allergies.add(allergy);
                }
            }
        }

        //辅检
        List<KlRegularConfig> rulesTepy = EntityUtil.makeEntityListMap(regularConfigs, "rulesTepy").get(RegularConfigEnum.pacs.getKey());
        Map<String, String> rulePacsCompiles = ruleAllCompile(rulesTepy, pacs);
        if (MapUtils.isNotEmpty(rulePacsCompiles)) {
            if (ListUtil.isNotEmpty(pacsNumList)) {
                ListIterator<PacsNum> pacsNumListIterator = pacsNumList.listIterator();
                while (pacsNumListIterator.hasNext()) {
                    PacsNum x = pacsNumListIterator.next();
                    String name = x.getName();
                    PD pd = x.getPd();
                    if (pd != null) {
                        String value = pd.getValue();
                        String key = name + "_" + value;
                        if (rulePacsCompiles.containsKey(key)) {
                            if (StringUtil.isNotBlank(rulePacsCompiles.get(key))) {
                                pd.setUnit(rulePacsCompiles.get(key));
                            }
                            rulePacsCompiles.remove(key);
                        }
                    }
                }
            }
            if (MapUtils.isNotEmpty(rulePacsCompiles)) {
                rulePacsCompiles.forEach((x, y) -> {
                    String[] nameValue = x.split("_");
                    PacsNum pacsNum = new PacsNum();
                    pacsNum.setName(nameValue[0]);
                    pacsNum.setStandName(nameValue[0]);
                    PD pdnew = new PD();
                    pdnew.setValue(nameValue[1]);
                    pdnew.setUnit(y);
                    pacsNum.setPd(pdnew);
                    pacsNumList.add(pacsNum);
                });
            }
        }

        //体征
        List<KlRegularConfig> rulesTepyVitals = EntityUtil.makeEntityListMap(regularConfigs, "rulesTepy").get(RegularConfigEnum.vital.getKey());
        Map<String, String> rulevitalsCompiles = ruleAllCompile(rulesTepyVitals, vital);
        if (MapUtils.isNotEmpty(rulevitalsCompiles)) {
            if (ListUtil.isNotEmpty(vitals)) {
                Iterator<Vital> vitalIterator = vitals.listIterator();
                while (vitalIterator.hasNext()) {
                    Vital next = vitalIterator.next();
                    String name = next.getName();
                    PD pd = next.getPd();
                    if (pd != null) {
                        String value = pd.getValue();
                        String key = name + "_" + value;
                        if (rulevitalsCompiles.containsKey(key)) {
                            if (StringUtil.isNotBlank(rulevitalsCompiles.get(key))) {
                                pd.setUnit(rulevitalsCompiles.get(key));
                            }
                            rulevitalsCompiles.remove(key);
                        }
                    }
                }
            }
            if (MapUtils.isNotEmpty(rulevitalsCompiles)) {
                rulevitalsCompiles.forEach((x, y) -> {
                    String[] nameValue = x.split("_");
                    Vital pacsNum = new Vital();
                    pacsNum.setName(nameValue[0]);
                    pacsNum.setStandName(nameValue[0]);
                    PD pdnew = new PD();
                    pdnew.setValue(nameValue[1]);
                    pdnew.setUnit(y);
                    pacsNum.setPd(pdnew);
                    vitals.add(pacsNum);
                });
            }
        }
        // 既往史过敏史
        List<RegularValueDTO> regularConfigPastss = getRegularConfigDatas(pasts,
                EntityUtil.makeEntityListMap(regularConfigs, "rulesTepy").get(RegularConfigEnum.allergy.getKey()));
        if (ListUtil.isNotEmpty(regularConfigPastss)) {
            for (RegularValueDTO dataPasts : regularConfigPastss) {
                AllergyMedicine allergy = new AllergyMedicine();
                allergy.setName(dataPasts.getKey());
                allergy.setStandName(dataPasts.getStandName());
                allergies.add(allergy);
            }
        }
        if (null != allergyStr && !allergyStr.equals("")) {
            String[] result = allergyStr.split("[\\,\\，]");
            for (String r : result) {
                AllergyMedicine allergyMedicine = new AllergyMedicine();
                allergyMedicine.setName(r.trim());
                allergyMedicine.setStandName(r.trim());
                allergies.add(allergyMedicine);

            }
        }
        pastLabel.setAllergyMedicines(allergies);
        wordCrfDTO.setPastLabel(pastLabel);
        wordCrfDTO.setLis(lisData);
        return wordCrfDTO;
    }

    public boolean saverRegularConfigDatas(List<RegularConfigSaveVO> regularConfigSaveVOList) {
        boolean res = false;
        for (RegularConfigSaveVO regularConfigSaveVO : regularConfigSaveVOList) {


            List<String> name = regularConfigSaveVO.getName();
            for (String str : name) {
                int countTow = this.count(new QueryWrapper<KlRegularConfig>()
                        .eq("is_deleted", IsDeleteEnum.N.getKey())
                        .eq(str != null, "rules_name", str)
                        .eq(regularConfigSaveVO.getStandardName() != null, "stand_name", str)
                        .eq(regularConfigSaveVO.getRulesTepy() != null, "rules_tepy", regularConfigSaveVO.getRulesTepy()));
                if (countTow > 0) {
                    throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, str + "该方案编码已存在");
                } else {
                    KlRegularConfig regularConfig = new KlRegularConfig();
                    regularConfig.setStandName(regularConfigSaveVO.getStandardName());
                    regularConfig.setRulesName(str);
                    regularConfig.setRulesTepy(regularConfigSaveVO.getRulesTepy());
                    String rulesValue = "";
                    if (regularConfigSaveVO.getRulesTepy().equals(RegularConfigEnum.allergy.getKey())) {
                        //int num = str.length();
                        rulesValue = "(\\、" + str + "过敏)|(无" + str + "过敏)|(否认" + str + "过敏)￥(" + str + "过敏)";
                    } else if (regularConfigSaveVO.getRulesTepy().equals(RegularConfigEnum.lis.getKey())) {
                        rulesValue = "(" + str + ")\\s*[(\\-|\\+)?\\d+(\\.\\d+)?$]+";
                    }
                    regularConfig.setRulesValue(rulesValue);
                    res = save(regularConfig);
                }
            }
        }
        return res;
    }

    /**
     * 把正则匹配出来的放在map里
     * key:体温_37 value:℃
     *
     * @param regexGroupData
     * @return
     */
    public Map<String, String> ruleCompile(List<List<String>> regexGroupData) {
        Map<String, String> ruleCompileMap = Maps.newHashMap();
        if (ListUtil.isNotEmpty(regexGroupData)) {
            for (List<String> groupContents : regexGroupData) {
                if (ListUtil.isNotEmpty(groupContents)) {
                    String s = "";
                    if (groupContents.size() > 2) {
                        s = groupContents.get(2);
                    }

                    ruleCompileMap.put(groupContents.get(0) + "_" + groupContents.get(1), s);
                }
            }

        }
        return ruleCompileMap;
    }

    public Map<String, String> ruleAllCompile(List<KlRegularConfig> rulesTepy, String txt) {
        Map<String, String> ruleAllCompileMap = Maps.newHashMap();
        if (ListUtil.isNotEmpty(rulesTepy)) {
            for (KlRegularConfig klrc : rulesTepy) {
                String rulesValue = klrc.getRulesValue();
                List<List<String>> regexGroupData = RegexUtil.getRegexGroupData(txt, rulesValue);
                Map<String, String> ruleCompileMap = ruleCompile(regexGroupData);
                if (MapUtils.isNotEmpty(ruleCompileMap)) {
                    ruleAllCompileMap.putAll(ruleCompileMap);
                }
            }
        }
        return ruleAllCompileMap;
    }
}
