package com.diagbot.facade;

import com.diagbot.dto.BaseDiagnoseDTO;
import com.diagbot.dto.BaseRegulationDTO;
import com.diagbot.dto.RegulationDTO;
import com.diagbot.dto.ScaleDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.rule.AgeRule;
import com.diagbot.rule.CommonRule;
import com.diagbot.rule.GroupRule;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.VitalRule;
import com.diagbot.service.impl.KlRegulationBaseServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RedisUtil;
import com.diagbot.vo.RegulationVO;
import com.diagbot.vo.ScaleVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Description:
 * @author: zhoutg
 * @time: 2021/3/15 10:58
 */
@Component
public class KlRegulationBaseFacade extends KlRegulationBaseServiceImpl {

    @Autowired
    RedisUtil redisUtil;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    CommonRule commonRule;
    @Autowired
    VitalRule vitalRule;
    @Autowired
    GroupRule groupRule;
    @Autowired
    LisRule lisRule;
    @Autowired
    AgeRule ageRule;
    @Autowired
    KlRegulationFacade klRegulationFacade;

    /**
     * 匹配规则
     *
     * @param scaleVO
     * @return
     */
    public ScaleDTO process(ScaleVO scaleVO) {
        ScaleDTO scaleDTO = new ScaleDTO();
        if (ListUtil.isEmpty(scaleVO.getCodeList())) {
            return scaleDTO;
        }
        // 筛选编码对应的规则
        List<BaseRegulationDTO> baseRegulationDTOList = this.getSelectBase(scaleVO.getCodeList());
        if (ListUtil.isEmpty(baseRegulationDTOList)) {
            return scaleDTO;
        }
        List<BaseDiagnoseDTO> baseDiagnoseDTOList = BeanUtil.listCopyTo(baseRegulationDTOList, BaseDiagnoseDTO.class);
        // CRF模型处理
        WordCrfDTO wordCrfDTO = commonFacade.crf_process(scaleVO, false);
        // 同义词转换
        commonFacade.wordStandConvert(wordCrfDTO);
        // 匹配的基础规则
        List<Long> baseIdList = commonRule.matchBase(wordCrfDTO, baseDiagnoseDTOList);

        scaleDTO.getDebug().put("基础规则匹配", StringUtils.join(baseIdList));

        if (ListUtil.isNotEmpty(baseIdList)) {
            RegulationVO regulationVO = new RegulationVO();
            regulationVO.setIds(baseIdList);
            List<RegulationDTO> regulationDTOList = klRegulationFacade.getRegulationByIds(regulationVO);
            if (ListUtil.isNotEmpty(regulationDTOList)) {
                scaleDTO.setCodeList(regulationDTOList.stream().map(r -> String.valueOf(r.getCode())).collect(Collectors.toList()));
            }
        }
        return scaleDTO;
    }
}
