package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.IndexBatchDTO;
import com.diagbot.entity.MappingConfig;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.MappingConfigService;
import com.diagbot.service.impl.MappingConfigServiceImpl;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ConceptVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/6/9 14:19
 */
@Component
public class MappingConfigFacade extends MappingConfigServiceImpl {

    private static Pattern pattern = Pattern.compile("\\s*|\t|\r|\n");

    @Autowired
    @Qualifier("mappingConfigServiceImpl")
    private MappingConfigService mappingConfigService;

    @Autowired
    private KlConceptFacade klConceptFacade;


    /**
     * 根据医院名称分组-返回id
     *
     * @param hisNames
     * @param type
     * @return
     */
    public Map<String, Map<String, Map<Long, List<Long>>>> groupByHisNameWithId(List<String> hisNames, Integer type, Long hospitalId) {
        Map<String, Map<String, Map<Long, List<Long>>>> retMap = new HashMap<>();
        QueryWrapper<MappingConfig> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("is_match", 1)
                .eq("type", type);
        if (hospitalId != null) {
            queryWrapper.eq("hospital_id", hospitalId);
        }
        if (ListUtil.isNotEmpty(hisNames)) {
            queryWrapper.in("his_name", hisNames);
        }
        List<MappingConfig> list = this.list(queryWrapper);

        if (ListUtil.isEmpty(list)) {
            return retMap;
        }

        for (MappingConfig item : list) {
            if (StringUtil.isBlank(item.getHisDetailName())) {
                item.setHisDetailName("");
            }
        }

        Map<String, List<MappingConfig>> hisMap = list.stream().collect(Collectors.groupingBy(MappingConfig::getHisName));
        for (Map.Entry<String, List<MappingConfig>> entry : hisMap.entrySet()) {
            Map<String, Map<Long, List<Long>>> subMap = new HashMap<>();
            Map<String, List<MappingConfig>> detailMap = entry.getValue().stream()
                    .collect(Collectors.groupingBy(MappingConfig::getHisDetailName));
            for (Map.Entry<String, List<MappingConfig>> subEntry : detailMap.entrySet()) {
                subMap.put(subEntry.getKey(), subEntry.getValue().stream().collect(Collectors.groupingBy(MappingConfig::getConceptId,
                        Collectors.mapping(MappingConfig::getFormConceptId, Collectors.toList()))));
            }
            retMap.put(entry.getKey(), subMap);
        }
        return retMap;
    }

    /**
     * 根据医院名称分组-返回名称
     *
     * @param hisNames
     * @param type
     * @param hospitalId
     * @return
     */
    public Map<String, Map<String, Map<String, List<String>>>> groupByHisNameWithName(List<String> hisNames, Integer type, Long hospitalId) {
        Map<String, Map<String, Map<String, List<String>>>> retMap = new HashMap<>();
        Map<String, Map<String, Map<Long, List<Long>>>> idMap = groupByHisNameWithId(hisNames, type, hospitalId);

        //conceptIds
        List<Long> ids = idMap.values()
                .stream()
                .flatMap(i -> i.values().stream())
                .map(i -> i.keySet())
                .flatMap(Collection::stream)
                .filter(i -> i != null)
                .distinct()
                .collect(Collectors.toList());

        //formConceptId
        ids.addAll(idMap.values()
                .stream()
                .flatMap(i -> i.values().stream())
                .flatMap(i -> i.values().stream())
                .flatMap(Collection::stream)
                .filter(i -> i != null)
                .distinct()
                .collect(Collectors.toList()));

        ConceptVO conceptVO = new ConceptVO();
        conceptVO.setSource(-1);
        conceptVO.setType(type);
        conceptVO.setIds(ids);
        List<IndexBatchDTO> respDTO = klConceptFacade.getConceptNames(conceptVO);
        List<IndexBatchDTO> indexList = respDTO;
        if (ListUtil.isNotEmpty(indexList)) {
            Map<Long, List<IndexBatchDTO>> indexMap
                    = indexList.stream().collect(Collectors.groupingBy(IndexBatchDTO::getId));
            for (Map.Entry<String, Map<String, Map<Long, List<Long>>>> entry : idMap.entrySet()) {
                Map<String, Map<String, List<String>>> subMap = new HashMap<>();
                for (Map.Entry<String, Map<Long, List<Long>>> subEntry : entry.getValue().entrySet()) {
                    Map<String, List<String>> thirdMap = new HashMap<>();
                    List<String> uniqueNames = Lists.newArrayList();
                    List<String> forms = Lists.newArrayList();
                    for (Long conceptId : subEntry.getValue().keySet()) {
                        uniqueNames.addAll(indexMap.get(conceptId).stream().map(IndexBatchDTO::getName).collect(Collectors.toList()));
                    }
                    List<Long> formConceptIds = subEntry.getValue().values()
                            .stream()
                            .flatMap(Collection::stream)
                            .distinct()
                            .collect(Collectors.toList());
                    for (Long formConceptId : formConceptIds) {
                        if (formConceptId == null) {
                            forms.add("");
                        } else {
                            forms.addAll(indexMap.get(formConceptId).stream().map(IndexBatchDTO::getName).collect(Collectors.toList()));
                        }
                    }
                    forms = forms.stream().distinct().collect(Collectors.toList());
                    for (String uniqueName : uniqueNames) {
                        thirdMap.put(uniqueName, forms);
                    }

                    subMap.put(subEntry.getKey(), thirdMap);
                }
                retMap.put(entry.getKey(), subMap);
            }
        }
        return retMap;
    }


}