package com.diagbot.process;

import com.diagbot.dto.BillMsg;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleConditionDTO;
import com.diagbot.dto.RuleExtDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.ConEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.RuleTypeEnum;
import com.diagbot.facade.CommonFacade;
import com.diagbot.facade.NeoFacade;
import com.diagbot.rule.AgeRule;
import com.diagbot.rule.BillRule;
import com.diagbot.rule.CommonRule;
import com.diagbot.rule.DrugRule;
import com.diagbot.rule.GroupRule;
import com.diagbot.rule.LisRule;
import com.diagbot.rule.MedEquRule;
import com.diagbot.rule.PacsRule;
import com.diagbot.rule.SexRule;
import com.diagbot.rule.VitalRule;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.CoreUtil;
import com.diagbot.vo.RuleVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

/**
 * @Description: 开单总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class BillProcess {

    @Autowired
    NeoFacade neoFacade;
    @Autowired
    DrugRule drugRule;
    @Autowired
    SexRule sexRule;
    @Autowired
    AgeRule ageRule;
    @Autowired
    CommonRule commonRule;
    @Autowired
    LisRule lisRule;
    @Autowired
    PacsRule pacsRule;
    @Autowired
    VitalRule vitalRule;
    @Autowired
    GroupRule groupRule;
    @Autowired
    MedEquRule medEquRule;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    BillRule billRule;

    // 规则处理
    public void processRule(List<RuleVO> ruleVOList, WordCrfDTO wordCrfDTO, IndicationDTO indicationDTO) {
        List<BillMsg> billMsgList = indicationDTO.getBillMsgList();
        Set<String> set = new LinkedHashSet<>();    // A与B不宜同时进行, B与A不宜同时进行 只能提示一个
        for (RuleVO ruleVO : ruleVOList) {
            /** 1、获取开单项相关数据 */
            RuleExtDTO ruleExtDTO = commonFacade.getRuleData(ruleVO);
            if (ruleExtDTO == null) {
                continue;
            }

            /** 2、简化拼接对象，用于开单项提示语 */
            RuleSimpleDTO ruleSimpleDTO = new RuleSimpleDTO();
            BeanUtil.copyProperties(ruleExtDTO, ruleSimpleDTO);

            /** 3、规则处理 */
            RuleTypeEnum ruleTypeEnum = RuleTypeEnum.getEnum(ruleExtDTO.getRuleType());
            List<RuleConditionDTO> ruleConditionDTOList = ruleExtDTO.getRuleConditionDTOList();
            for (RuleConditionDTO ruleConditionDTO : ruleConditionDTOList) {
                CoreUtil.resetRuleSimple(ruleSimpleDTO); // 重置
                ruleSimpleDTO.setMsg(ruleConditionDTO.getMsg());
                List<RuleBaseDTO> ruleBaseDTOList = ruleConditionDTO.getRuleBaseDTOList();
                switch (ruleTypeEnum) {
                    case bill: // redis 数据以 _1结尾
                        billRule.bill(wordCrfDTO, ruleBaseDTOList, billMsgList, ruleSimpleDTO, set);
                        break;
                    case repeat:  // redis 数据以 _6结尾
                        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
                            switch (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType())) {
                                case PacsName: // 辅助检查项目
                                case PacsSubName: // 辅助检查子项目
                                    // 辅检正常值，无需重复开单项
                                    commonRule.needlessRepeatOrderPacs(wordCrfDTO, ruleBaseDTO, billMsgList, ConEnum.repeat.getName(), ruleSimpleDTO);
                                    break;
                                case LisSubName: // 化验结果
                                    // 化验正常值，无需重复开单项
                                    commonRule.needlessRepeatOrderLis(wordCrfDTO, ruleBaseDTO, billMsgList, ConEnum.repeat.getName(), ruleSimpleDTO);
                                    break;
                            }
                        }
                        break;
                    default:
                        break;
                }
            }
        }
        // 24小时重复开单项
        commonRule.repeat24Bill(wordCrfDTO, billMsgList);
    }
}
