package com.diagbot.rule;

import com.diagbot.dto.BillMsg;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.ConEnum;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.MsgUtil;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @description: 年龄规则
 * @author: zhoutg
 * @time: 2020/8/3 14:47
 */
@Component
public class AgeRule {

    /**
     * 比较年龄——开单项
     *
     * @param wordCrfDTO
     * @param ruleBaseDTO
     * @param ruleSimpleDTO
     */
    public Boolean bill(WordCrfDTO wordCrfDTO, RuleBaseDTO ruleBaseDTO, RuleSimpleDTO ruleSimpleDTO) {
        Boolean flag = getFlag(wordCrfDTO, ruleBaseDTO);
        if (flag) {
            List<String> contentList = Lists.newArrayList();
            if (!contentList.contains(wordCrfDTO.getAge())) {
                contentList.add(wordCrfDTO.getAge());
            }
            CoreUtil.setListAndType(ruleSimpleDTO, contentList, ConEnum.age.getName());
        }
        return flag;
    }

    /**
     * 比较年龄——推送
     *
     * @param wordCrfDTO
     * @param ruleBaseDTO
     * @param baseIdList
     * @param ids
     */
    public void push(WordCrfDTO wordCrfDTO, RuleBaseDTO ruleBaseDTO, List<Long> baseIdList, String ids) {
        Boolean flag = getFlag(wordCrfDTO, ruleBaseDTO);
        if (flag) {
            CoreUtil.addSplitString(baseIdList, ids);
        }
    }

    /**
     * 比较年龄——高危手术
     *
     * @param wordCrfDTO
     * @param ruleBaseDTO
     * @param highRiskList
     * @param ruleSimpleDTO
     */
    public void highOperation(WordCrfDTO wordCrfDTO, RuleBaseDTO ruleBaseDTO, List<BillMsg> highRiskList, RuleSimpleDTO ruleSimpleDTO) {
        Boolean flag = getFlag(wordCrfDTO, ruleBaseDTO);
        if (flag) {
            String message = "年龄:" + wordCrfDTO.getAge();
            BillMsg billMsg = MsgUtil.getCommonHighRiskMsg(message, ruleSimpleDTO.getInputName(), ruleSimpleDTO.getLibTypeName());
            highRiskList.add(billMsg);
        }
    }

    /**
     * 匹配规则
     *
     * @param wordCrfDTO
     * @param ruleBaseDTO
     * @return
     */
    public boolean getFlag(WordCrfDTO wordCrfDTO, RuleBaseDTO ruleBaseDTO) {
        Boolean flag = false;
        if (wordCrfDTO.getAgeNum() != null) {
            Double age = wordCrfDTO.getAgeNum();
            flag = CoreUtil.compareNum(ruleBaseDTO, age);
        }
        return flag;
    }

}
