package com.diagbot.rule;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.BaseTypeEnum;
import com.diagbot.enums.ConEnum;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.facade.CommonFacade;
import com.diagbot.model.entity.Clinical;
import com.diagbot.model.entity.Operation;
import com.diagbot.util.ListUtil;
import com.diagbot.util.MsgNewUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Set;

/**
 * @description: 开单规则
 * @author: zhoutg
 * @time: 2020/8/3 14:47
 */
@Component
public class BillRule {

    @Autowired
    SexRule sexRule;
    @Autowired
    AgeRule ageRule;
    @Autowired
    DeptRule deptRule;
    @Autowired
    MsgNewUtil msgNewUtil;
    @Autowired
    CommonRule commonRule;
    @Autowired
    LisRule lisRule;
    @Autowired
    PacsRule pacsRule;
    @Autowired
    VitalRule vitalRule;
    @Autowired
    GroupRule groupRule;
    @Autowired
    MedEquRule medEquRule;
    @Autowired
    CommonFacade commonFacade;
    @Autowired
    DrugRule drugRule;
    @Autowired
    RouteRule routeRule;
    @Autowired
    FormRule formRule;

    /**
     * 开单规则总入口
     *
     * @param wordCrfDTO
     * @param ruleBaseDTOList
     * @param billMsgList
     * @param ruleSimpleDTO
     */
    public void bill(WordCrfDTO wordCrfDTO, List<RuleBaseDTO> ruleBaseDTOList, List<BillMsg> billMsgList, RuleSimpleDTO ruleSimpleDTO, Set<String> set) {
        if (ListUtil.isEmpty(ruleBaseDTOList)) {
            return;
        }
        /**---------------------------数据统一处理开始--------------------------------- */
        // 【过敏药品数据来源】
        List<Item> allergyMedicines = commonFacade.getDrugAllergySource(wordCrfDTO);
        // 【药品数据来源】(主诉、现病史、既往史、结构化药品)
        List<Item> medicineAll = wordCrfDTO.getDrugSource();
        // 【诊断数据来源】
        List<Item> diags = wordCrfDTO.getDiagSource();
        // 【辅检结果数据来源】
        List<Item> pacsDescList = wordCrfDTO.getPacsLabel().getRes();
        // 【手术数据来源】
        List<Operation> operationsSource = commonFacade.getOperationSource(wordCrfDTO);
        // 【临床表现数据来源】（主诉、现病史）
        List<Clinical> clinicals = commonFacade.getClinicalSource(wordCrfDTO);
        /**---------------------------数据统一处理结束--------------------------------- */

        for (RuleBaseDTO ruleBaseDTO : ruleBaseDTOList) {
            boolean flag = false;
            boolean flag1 = false, flag2 = false; // 定义多个变量
            if (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType()) != null) { // 避免空指针
                switch (LexiconEnum.getEnum(ruleBaseDTO.getBaseLibType())) {
                    case Gender: // 性别
                        flag = sexRule.bill(wordCrfDTO, ruleBaseDTO, ruleSimpleDTO);
                        break;
                    case Age: // 年龄
                        flag = ageRule.bill(wordCrfDTO, ruleBaseDTO, ruleSimpleDTO);
                        break;
                    case Dept: // 科室
                        flag = deptRule.bill(wordCrfDTO, ruleBaseDTO, ruleSimpleDTO);
                        break;
                    case Disease: // 诊断
                        flag = commonRule.compareItemWithBill(diags, ruleBaseDTO, ruleSimpleDTO, ConEnum.disease.getName());
                        break;
                    case LisName: // 化验开单互斥
                        flag1 = commonRule.exclusionBillOrder(wordCrfDTO.getLisOrder(), ruleBaseDTO, ruleSimpleDTO, ConEnum.exclusion.getName(), set);
                        flag2 = commonRule.exclusionBillStruct(wordCrfDTO.getLis(), ruleBaseDTO, ruleSimpleDTO, ConEnum.exclusion.getName(), set);
                        if (flag1 || flag2) {
                            flag = true;
                        }
                        break;
                    case LisSubName: // 化验结果
                        flag = lisRule.bill(wordCrfDTO.getLis(), ruleBaseDTO, ruleSimpleDTO);
                        break;
                    case VitalResult: //体格检查结果
                    case Vital: // 体格检查项目
                        flag = vitalRule.bill(wordCrfDTO.getVitalLabel(), ruleBaseDTO, ruleSimpleDTO);
                        break;
                    case PacsName: // 辅助检查项目
                    case PacsSubName: // 辅助检查子项目
                        if (ruleBaseDTO.getBaseType() != null && ruleBaseDTO.getBaseType().equals(BaseTypeEnum.B5.getKey())) {
                            // 开单项互斥
                            flag1 = commonRule.exclusionBillOrder(wordCrfDTO.getPacsOrder(), ruleBaseDTO, ruleSimpleDTO, ConEnum.exclusion.getName(), set);
                            flag2 = commonRule.exclusionBillStruct(wordCrfDTO.getPacs(), ruleBaseDTO, ruleSimpleDTO, ConEnum.exclusion.getName(), set);
                            if (flag1 || flag2) {
                                flag = true;
                            }
                        } else {
                            // 辅检项目,暂无数据
                            flag = pacsRule.bill(wordCrfDTO.getPacs(), ruleBaseDTO, ruleSimpleDTO, ConEnum.pacs.getName());
                        }
                        break;
                    case Symptom: // 临床表现
                        flag = commonRule.compareNameWithBill(clinicals, ruleBaseDTO, ruleSimpleDTO, ConEnum.clinicfindings.getName());
                        break;
                    case Operation: // 手术(既往史、现病史、医嘱手术)
                        flag = commonRule.compareNameWithBill(operationsSource, ruleBaseDTO, ruleSimpleDTO, ConEnum.operations.getName());
                        break;
                    case Medicine: // 药品通用名
                    case MedChemClass: // 药品化学物质类别
                    case MedZhiLiaoClass: // 药品治疗学类别
                    case MedYaoLiClass: // 药品药理学类别
                    case MedJiePouClass: // 药品解剖学类别
                        if (ruleBaseDTO.getBaseType().equals(4)) {
                            // 禁忌过敏药品（既往史）
                            flag = drugRule.bill(allergyMedicines, ruleBaseDTO, ruleSimpleDTO, ConEnum.allergicmeds.getName());
                        } else {
                            // 服用药品（主诉、现病史、既往史、结构化药品）
                            flag = drugRule.bill(medicineAll, ruleBaseDTO, ruleSimpleDTO, ConEnum.oralmeds.getName());
                        }
                        break;
                    case Allergen: // 其他过敏原
                        flag = drugRule.bill(allergyMedicines, ruleBaseDTO, ruleSimpleDTO, ConEnum.otherAllergy.getName());
                        break;
                    case Group: // 禁忌人群
                        flag = groupRule.bill(wordCrfDTO, ruleBaseDTO, ruleSimpleDTO);
                        break;
                    case PacsResult: // 禁用辅助检查结果
                        flag = commonRule.compareItemWithBill(pacsDescList, ruleBaseDTO, ruleSimpleDTO, ConEnum.pacsDesc.getName());
                        break;
                    case Device: // 禁忌医疗器械及物品
                        flag = medEquRule.bill(wordCrfDTO, ruleBaseDTO, ruleSimpleDTO);
                        break;
                    case AdministrationRoute: // 给药途径
                        flag = routeRule.bill(ruleBaseDTO, ruleSimpleDTO);
                        break;
                    case Form: // 药品剂型
                        flag = formRule.bill(ruleBaseDTO, ruleSimpleDTO);
                        break;
                    default:
                        break;
                }
            }
            // 有任一条件不满足，直接退出
            if (!flag) {
                return;
            }
        }

        // 提示语处理
        for (String content : ruleSimpleDTO.getContentList()) {
            ruleSimpleDTO.setContent(content);
            msgNewUtil.getCommonBillMsg(ruleSimpleDTO, billMsgList);
        }
    }

}
