package com.diagbot.rule;

import com.diagbot.dto.RuleBaseDTO;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.enums.LexiconEnum;
import com.diagbot.enums.RedisEnum;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RedisUtil;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

/**
 * @description: 实验室检查和辅助检查规则
 * @author: kwz
 * @time: 2020/8/3 14:47
 */
@Component
public class DrugRule {

    @Autowired
    RedisUtil redisUtil;

    /**
     * 比较药品是否匹配(先根据baseLibType判断返回的是药品通用名还是药品分类，如果返回的是药品分类，从redis获取药品通用名)
     *
     * @param inputList
     * @param ruleBaseDTO
     * @param ruleSimpleDTO
     * @param <T>
     */
    public <T> Boolean bill(List<T> inputList, RuleBaseDTO ruleBaseDTO, RuleSimpleDTO ruleSimpleDTO, String conType) {
        boolean flag = false;
        List<String> contentList = Lists.newArrayList();
        if (ListUtil.isNotEmpty(inputList)) {
            List<String> allDrug = new ArrayList<>();
            if (ruleBaseDTO.getBaseLibType().equals(LexiconEnum.MedChemClass.getKey())
                    || ruleBaseDTO.getBaseLibType().equals(LexiconEnum.MedZhiLiaoClass.getKey())
                    || ruleBaseDTO.getBaseLibType().equals(LexiconEnum.MedYaoLiClass.getKey())
                    || ruleBaseDTO.getBaseLibType().equals(LexiconEnum.MedJiePouClass.getKey())) {
                List<String> drugRedis = redisUtil.get(RedisEnum.drugType.getName() + ruleBaseDTO.getBaseLibName());
                if (ListUtil.isNotEmpty(drugRedis)) {
                    allDrug.addAll(drugRedis);
                }
            } else {
                allDrug.add(ruleBaseDTO.getBaseLibName());
            }
            for (T d : inputList) {
                String uniqueName = (String) CoreUtil.getFieldValue(d, "uniqueName");
                String name = (String) CoreUtil.getFieldValue(d, "name");
                if (StringUtils.isNotBlank(uniqueName) && allDrug.contains(uniqueName)) {
                    if (!contentList.contains(name)) {
                        contentList.add(name);
                    }
                    flag = true;
                }
            }
        }
        if (flag) {
            CoreUtil.setListAndType(ruleSimpleDTO, contentList, conType);
        }
        return flag;
    }
}
