package com.diagbot.util;

import com.diagbot.biz.push.entity.Lis;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.RuleSimpleDTO;
import com.diagbot.enums.CommonEnum;
import com.diagbot.enums.ConEnum;
import com.diagbot.enums.RedisEnum;
import com.diagbot.enums.TypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @description: 提示语工具类
 * @author: zhoutg
 * @time: 2020/9/14 11:16
 */
@Component
public class MsgNewUtil {

    /**
     * 占位符静态变量，格式：“前缀#后缀”
     */
    public static Map<String, String> placeholderMap = new ConcurrentHashMap<>();

    @Autowired
    RedisUtil redisUtil;

    /**
     * 开单合理性通用提示信息
     *
     * @param ruleSimpleDTO
     * @return
     */
    public void getCommonBillMsg(RuleSimpleDTO ruleSimpleDTO, List<BillMsg> billMsgList) {
        String orderName = ruleSimpleDTO.getInputName(); // 医院名称
        String orderStandName = ruleSimpleDTO.getLibName(); // 标准名称
        String orderType = ruleSimpleDTO.getLibTypeName(); // 开单项类型
        String content = ruleSimpleDTO.getContent(); // 界面匹配内容
        String conType = ruleSimpleDTO.getConType(); // 禁忌类型
        String unionMsg = ruleSimpleDTO.getMsg(); // 原因及建议
        String structName = ruleSimpleDTO.getStructName(); // 项目名称
        String msg = redisUtil.getByKeyAndField(RedisEnum.msg.getName(), orderType + "_" + conType);
        if (StringUtil.isBlank(msg)) {
            // msg = redisUtil.getByKeyAndField(RedisEnum.msg.getName(), "通用" + "_" + "默认提示语");
            msg = "";
        }
        BillMsg billMsg = new BillMsg();
        // 出现2个“具有”，去除一个为空
        if (ConEnum.medEqu.getName().equals(conType) && StringUtil.isNotBlank(content)
                && content.startsWith("具有") && msg.startsWith("该患者具有")) {
            msg = msg.replace("具有", "");
        }
        // 给药途径替换
        if (ruleSimpleDTO.isRouteMatch()) {
            msg = msg.replace("[给药途径]", ruleSimpleDTO.getRouteName());
        } else {
            msg = msg.replace("[给药途径]", "");
        }
        if (StringUtil.isNotBlank(unionMsg)) {
            msg = msg.replace("[原因及建议]", "，" + unionMsg);
        } else {
            msg = msg.replace("[原因及建议]", "");
        }

        // [性别取反]标签替换
        if (ConEnum.gender.getName().equals(conType)) {
            if ("男".equals(content)) {
                msg = msg.replace("[性别取反]", "女");
            } else if ("女".equals(content)) {
                msg = msg.replace("[性别取反]", "男");
            }
        }

        // 放到最后统一替换【多处匹配】
        msg = msg.replace("[匹配内容]", content);
        msg = msg.replace("[开单名称]", orderName);
        msg = msg.replace("[项目名称]", structName);

        billMsg.setMsg(msg);
        billMsg.setOrderName(orderName);
        billMsg.setOrderStandName(orderStandName);
        billMsg.setContent(content);
        billMsg.setType(conType);

        CoreUtil.addBeanToList(billMsgList, billMsg);
    }

    /**
     * 其他值通用提示信息
     *
     * @param content
     * @param name
     * @param type
     * @param lis
     * @param suggestMsg
     * @return
     */
    public BillMsg getCommonOtherMsg(String otherType, String content, String name, String type, Lis lis, String suggestMsg, String lisNameDetail) {
        BillMsg billMsg = new BillMsg();
        String msg = "";
        switch (TypeEnum.getEnum(otherType)) {
            case lis: // 其他值提醒——化验
                msg = redisUtil.getByKeyAndField(RedisEnum.otherMsg.getName(), otherType + "_" + type);
                if (StringUtil.isBlank(msg)) { // 防止报空指针
                    return null;
                }
                if (StringUtil.isNotBlank(suggestMsg)) {
                    msg = msg.replace("[原因及建议]", "，" + suggestMsg);
                } else {
                    msg = msg.replace("[原因及建议]", "");
                }
                // 放到最后统一替换【多处匹配】
                msg = msg.replace("[化验结果]", content);
                msg = msg.replace("[匹配内容]", name);
                if (msg.contains("[化验套餐和明细]")) {
                    msg = msg.replace("[化验套餐和明细]", lisNameDetail);
                }
                billMsg.setType(type);
                billMsg.setOrderName(lis.getName());
                billMsg.setOrderDetailName(lis.getDetailName());
                break;
            default:
                break;
        }
        billMsg.setMsg(msg);
        billMsg.setContent(content);
        return billMsg;
    }

    /**
     * 其他值提醒辅检提示信息
     *
     * @param mainMsg
     * @param templateType
     * @param suggestMsg
     * @return
     */
    public BillMsg getOtherPacsMsg(String mainMsg, String templateType, String suggestMsg) {
        String msg = redisUtil.getByKeyAndField(RedisEnum.otherMsg.getName(), TypeEnum.pacs.getName() + "_" + templateType);
        if (StringUtil.isBlank(msg)) { // 防止报空指针
            return null;
        }
        BillMsg billMsg = new BillMsg();
        if (StringUtil.isNotBlank(suggestMsg)) {
            msg = msg.replace("[原因及建议]", "，" + suggestMsg);
        } else {
            msg = msg.replace("[原因及建议]", "");
        }
        // 放到最后统一替换【多处匹配】
        msg = msg.replace("[辅助检查描述]", mainMsg);
        billMsg.setType(templateType);
        billMsg.setMsg(msg);
        billMsg.setContent(mainMsg);
        return billMsg;
    }

    /**
     * 危急值化验提示信息
     *
     * @param type          分类
     * @param content       化验内容
     * @param name          条件内容
     * @param conditionType 条件类型
     * @param suggestMsg
     * @return
     */
    public BillMsg getCriticalMsg(String type, String content, String name, String conditionType, String suggestMsg) {
        BillMsg billMsg = new BillMsg();
        String msg = "";
        switch (TypeEnum.getEnum(type)) {
            case lis: // 危急值提醒——化验
                msg = redisUtil.getByKeyAndField(RedisEnum.criticalMsg.getName(), type + "_" + conditionType);
                if (StringUtil.isBlank(msg)) { // 防止报空指针
                    return null;
                }
                if (StringUtil.isNotBlank(suggestMsg)) {
                    msg = msg.replace("[原因及建议]", "，" + suggestMsg);
                } else {
                    msg = msg.replace("[原因及建议]", "");
                }
                // 放到最后统一替换【多处匹配】
                msg = msg.replace("[化验结果]", content);
                msg = msg.replace("[匹配内容]", name);
                billMsg.setType(conditionType);
                break;
            default:
                break;
        }
        billMsg.setMsg(msg);
        billMsg.setContent(content);
        return billMsg;
    }

    /**
     * 不使用正则，全部替换
     *
     * @param msg         文本
     * @param placeholder 占位符
     * @param replaceStr  替换字符串
     * @return
     */
    public String replacePlaceholder(String msg, String placeholder, String replaceStr) {
        if (StringUtil.isBlank(msg)) {
            return null;
        }
        if (StringUtil.isBlank(placeholder)) {
            return msg;
        }
        if (msg.contains(placeholder)) {
            String prefixAndSuffix = placeholderMap.get(placeholder); // 前后缀，中间用#分隔
            String prefix = ""; // 前缀
            String suffix = ""; // 后缀
            if (StringUtil.isNotBlank(prefixAndSuffix)) {
                String[] split = prefixAndSuffix.split(CommonEnum.splitSymbol.getName());
                prefix = split[0];
                if (split.length == 2) {
                    suffix = split[1];
                } else {
                    suffix = "";
                }
            }
            // 拼接前缀
            if (StringUtil.isNotBlank(prefix)) {
                replaceStr = prefix + replaceStr;
            }
            // 拼接后缀
            if (StringUtil.isNotBlank(suffix)) {
                replaceStr = replaceStr + suffix;
            }
            // 后缀后面默认添加标点符号
            replaceStr = replaceStr + "，";
            msg = msg.replace(placeholder, replaceStr);
            // 提示语如果以，结尾，去掉
            if (msg.matches(".*[,|，]$")) {
                msg = msg.substring(0, msg.length() - 1);
            }
        }
        return msg;
    }
}
