package com.diagbot.web;

import com.diagbot.dto.PushNewDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.StandConvertCrfBatchDTO;
import com.diagbot.facade.PushFacade;
import com.diagbot.facade.TestFacade;
import com.diagbot.vo.PushNewVO;
import com.diagbot.vo.StandConvertCrfVO;
import com.diagbot.vo.TestCrfVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @Description: CDSS核心大数据接口API控制层
 * @author: zhoutg
 * @time: 2018/8/30 10:12
 */
@RestController
@RequestMapping("/bigDebug")
@Api(value = "大数据调试API", tags = { "大数据调试API" })
@SuppressWarnings("unchecked")
public class BigDebugController {

    @Autowired
    private TestFacade testFacade;
    @Autowired
    PushFacade pushFacade;

    @ApiOperation(value = "crf解析[zhoutg]", notes = "content：文本内容\n" +
            "modelType:模型类型,主诉、现病史：chief_present，既往史：PastFamily_cx，查体：Present_cx，个人史：PersonalHistory_cx，诊断：Diagnoses_cx")
    @PostMapping("/testCrf")
    public RespDTO<Object> testCrf(@RequestBody TestCrfVO testCrfVO) {
        return RespDTO.onSuc(testFacade.testCrf(testCrfVO));
    }

    @ApiOperation(value = "标准词批量转换API[zhoutg]", notes = "类型，疾病：disease，症状：symptom，手术和操作：operation，药品：drug，实验室检查：lis，辅助检查：pacs， 查体：vital")
    @PostMapping("/testStandConvertBatch")
    public RespDTO<StandConvertCrfBatchDTO> testStandConvertBatch(@RequestBody List<StandConvertCrfVO> standConvertCrfVOList) {
        return RespDTO.onSuc(testFacade.testStandConvertBatch(standConvertCrfVOList));
    }

    @ApiOperation(value = "大数据推送接口测试[zhoutg]", notes = "")
    @PostMapping("/testPush")
    public RespDTO<PushNewDTO> testPush(@RequestBody PushNewVO pushNewVO) {
        return RespDTO.onSuc(pushFacade.testPushFac(pushNewVO));
    }

    // @ApiOperation(value = "标准词转换API[zhoutg]", notes = "类型，疾病: disease，症状: symptom，手术和操作:operation，药品: drug，实验室检查：lis，辅助检查：pacs， 辅助检查：vital")
    // @PostMapping("/testStandConvert")
    // @ApiIgnore
    // public RespDTO<StandConvertCrfDTO> testStandConvert(@RequestBody StandConvertCrfVO standConvertCrfVO) {
    //     return RespDTO.onSuc(testFacade.testStandConvert(standConvertCrfVO));
    // }

    // @ApiOperation(value = "标准词批量转换【新版】API[zhoutg]", notes = "类型，疾病: disease，症状: symptom，手术和操作:operation，药品: drug，实验室检查：lis，辅助检查：pacs， 查体：vital")
    // @PostMapping("/testStandConvertBatchNew")
    // public RespDTO<StandConvertCrfBatchDTO> testStandConvertBatchNew(@RequestBody List<StandConvertCrfVO> standConvertCrfVOList) {
    //     return RespDTO.onSuc(testFacade.testStandConvertBatchNew(standConvertCrfVOList));
    // }
}

