package com.diagbot.web;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.ConceptDetailDTO;
import com.diagbot.dto.ConceptScaleDTO;
import com.diagbot.dto.KlConceptStaticDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.StaticKnowledgeDTO;
import com.diagbot.dto.StaticKnowledgeIndexPageDTO;
import com.diagbot.facade.KlConceptStaticFacade;
import com.diagbot.util.BeanUtil;
import com.diagbot.vo.ChangeStatusVO;
import com.diagbot.vo.GetDetailVO;
import com.diagbot.vo.IdVO;
import com.diagbot.vo.KlConceptStaticPageVO;
import com.diagbot.vo.KlConceptStaticVO;
import com.diagbot.vo.StaticKnowledgeIndexPageVO;
import com.diagbot.vo.StaticKnowledgeVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/2/24 11:34
 */
@RestController
@RequestMapping("/kl/conceptStatic")
@Api(value = "静态知识维护相关API", tags = { "静态知识维护相关API" })
@SuppressWarnings("unchecked")
public class KlConceptStaticController {
    @Autowired
    KlConceptStaticFacade klConceptStaticFacade;

    @ApiOperation(value = "获取静态知识[zhaops]",
            notes = "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作、8-量表、9-护理、10-政策法规 <br>" +
                    "contentTypes： 内容类型（多选）：1-静态信息、2-注意事项、3-临床路径、4-治疗方案<br>" +
                    "name： 标准术语名称<br>")
    @PostMapping("/getStaticKnowledge")
    @SysLogger("getStaticKnowledge")
    public RespDTO<StaticKnowledgeDTO> getStaticKnowledge(@Valid @RequestBody StaticKnowledgeVO staticKnowledgeVO) {
        StaticKnowledgeDTO data = klConceptStaticFacade.getStaticKnowledge(staticKnowledgeVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "获取静态知识列表[zhaops]",
            notes = "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作、8-量表、9-护理、10-政策法规  <br>" +
                    "name： 术语名称<br>" +
                    "status： 启用状态：1-启用、0-禁用<br>")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    @SuppressWarnings("unchecked")
    public RespDTO<Page<KlConceptStaticDTO>> getPage(@Valid @RequestBody KlConceptStaticPageVO klConceptStaticPageVO) {
        IPage<KlConceptStaticDTO> data = klConceptStaticFacade.getPage(klConceptStaticPageVO);
        Page<KlConceptStaticDTO> page = new Page<>();
        BeanUtil.copyProperties(data, page);
        return RespDTO.onSuc(page);
    }

    @ApiOperation(value = "静态知识检索（分页返回）[zhaops]",
            notes = "types： 类型（多选）：0-全部、1-诊断、2-药品、3-检验、5-检查、6-手术和操作、8-量表、9-护理、10-政策法规  <br>" +
                    "inputStr： 检索内容<br>")
    @PostMapping("/staticIndexPage")
    @SysLogger("staticIndexPage")
    @SuppressWarnings("unchecked")
    public RespDTO<Page<StaticKnowledgeIndexPageDTO>> staticIndexPage(@Valid @RequestBody StaticKnowledgeIndexPageVO staticKnowledgeIndexPageVO) {
        IPage<StaticKnowledgeIndexPageDTO> data = klConceptStaticFacade.staticIndexPage(staticKnowledgeIndexPageVO);
        Page<StaticKnowledgeIndexPageDTO> page = new Page<>();
        BeanUtil.copyProperties(data, page);
        return RespDTO.onSuc(page);
    }


    @ApiOperation(value = "保存静态知识-新增或修改[zhaops]",
            notes = "id： id <br>" +
                    "name： 术语名称 <br>" +
                    "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作、8-量表、9-护理、10-政策法规  <br>" +
                    "clinicalPathwayName： 临床路径名称<br>" +
                    "noticeName： 注意事项名称<br>" +
                    "source：数据来源：0-医院端、1-云平台，默认0 <br>" +
                    "details： 明细<br>" +
                    "scale：评估量表<br>")
    @PostMapping("/saveOrUpdateRecord")
    @SysLogger("saveOrUpdateRecord")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecord(@Valid @RequestBody KlConceptStaticVO klConceptStaticVO) {
        Boolean data = klConceptStaticFacade.saveOrUpdateRecord(klConceptStaticVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "静态知识启用禁用[zhaops]",
            notes = "id： 术语id <br>" +
                    "status： 启用状态（0：禁用，1：启用）<br>")
    @PostMapping("/changeStatus")
    @SysLogger("changeStatus")
    @Transactional
    public RespDTO<Boolean> changeStatus(@Valid @RequestBody ChangeStatusVO changeStatusVO) {
        Boolean data = klConceptStaticFacade.changeStatus(changeStatusVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据术语id获取静态信息[zhaops]",
            notes = "id： 术语id <br>")
    @PostMapping("/getRecordById")
    @SysLogger("getRecordById")
    public RespDTO<KlConceptStaticDTO> getRecordById(@Valid @RequestBody IdVO idVO) {
        KlConceptStaticDTO data = klConceptStaticFacade.getRecordById(idVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "静态知识是否存在[zhaops]",
            notes = "name： 术语名称 <br>" +
                    "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作、8-量表、9-护理、10-政策法规  <br>")
    @PostMapping("/isExist")
    @SysLogger("isExist")
    public RespDTO<Boolean> isExist(@Valid @RequestBody KlConceptStaticVO klConceptStaticVO) {
        Boolean data = klConceptStaticFacade.isExist(klConceptStaticVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "推理结果匹配静态知识[zhaops]",
            notes = "nameList： 术语名称 <br>" +
                    "contentTypes：明细内容")
    @PostMapping("/getDetailByConcept")
    @SysLogger("getDetailByConcept")
    public RespDTO<Map<String, List<ConceptDetailDTO>>> getDetailByConcept(@Valid @RequestBody GetDetailVO getDetailVO) {
        Map<String, List<ConceptDetailDTO>> data = klConceptStaticFacade.getDetailByConcept(getDetailVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "推理结果匹配量表[zhaops]",
            notes = "nameList： 术语名称 <br>")
    @PostMapping("/getScaleDetailByConcept")
    @SysLogger("getScaleDetailByConcept")
    public RespDTO<Map<String, ConceptScaleDTO>> getScaleDetailByConcept(@RequestBody @Valid GetDetailVO getDetailVO) {
        Map<String, ConceptScaleDTO> data = klConceptStaticFacade.getScaleDetailByConcept(getDetailVO);
        return RespDTO.onSuc(data);
    }
}